@@ -32,6 +32,7 @@
 #include <glib-object.h>
 #include <grl-metadata-key.h>
 #include <grl-definitions.h>
+#include <grl-related-keys.h>
 
 G_BEGIN_DECLS
 
@@ -59,7 +60,7 @@ G_BEGIN_DECLS
 #define GRL_DATA_GET_CLASS(obj)                 \
   (G_TYPE_INSTANCE_GET_CLASS ((obj),            \
                               GRL_TYPE_DATA,    \
-                              DataClass))
+                              GrlDataClass))
 
 typedef struct _GrlData        GrlData;
 typedef struct _GrlDataPrivate GrlDataPrivate;
@@ -117,7 +118,7 @@ gfloat grl_data_get_float (GrlData *data, GrlKeyID key);
 
 const guint8 *grl_data_get_binary(GrlData *data, GrlKeyID key, gsize *size);
 
-void grl_data_add (GrlData *data, GrlKeyID key);
+G_GNUC_DEPRECATED void grl_data_add (GrlData *data, GrlKeyID key);
 
 void grl_data_remove (GrlData *data, GrlKeyID key);
 
@@ -125,11 +126,39 @@ gboolean grl_data_has_key (GrlData *data, GrlKeyID key);
 
 GList *grl_data_get_keys (GrlData *data);
 
-gboolean grl_data_key_is_known (GrlData *data, GrlKeyID key);
+G_GNUC_DEPRECATED gboolean grl_data_key_is_known (GrlData *data, GrlKeyID key);
 
-void grl_data_set_overwrite (GrlData *data, gboolean overwrite);
+void grl_data_add_related_keys (GrlData *data, GrlRelatedKeys *relkeys);
 
-gboolean grl_data_get_overwrite (GrlData *data);
+void grl_data_add_string (GrlData *data, GrlKeyID key, const gchar *strvalue);
+
+void grl_data_add_int (GrlData *data, GrlKeyID key, gint intvalue);
+
+void grl_data_add_float (GrlData *data, GrlKeyID key, gfloat floatvalue);
+
+void grl_data_add_binary (GrlData *data, GrlKeyID key, const guint8 *buf, gsize size);
+
+guint grl_data_length (GrlData *data, GrlKeyID key);
+
+GrlRelatedKeys *grl_data_get_related_keys (GrlData *data, GrlKeyID key, guint index);
+
+G_GNUC_DEPRECATED GList *grl_data_get_all_single_related_keys (GrlData *data, GrlKeyID key);
+
+GList *grl_data_get_single_values_for_key (GrlData *data, GrlKeyID key);
+
+G_GNUC_DEPRECATED GList *grl_data_get_all_single_related_keys_string (GrlData *data, GrlKeyID key);
+
+GList *grl_data_get_single_values_for_key_string (GrlData *data, GrlKeyID key);
+
+void grl_data_remove_nth (GrlData *data, GrlKeyID key, guint index);
+
+void grl_data_set_related_keys (GrlData *data, GrlRelatedKeys *relkeys, guint index);
+
+GrlData *grl_data_dup (GrlData *data);
+
+G_GNUC_DEPRECATED void grl_data_set_overwrite (GrlData *data, gboolean overwrite);
+
+G_GNUC_DEPRECATED gboolean grl_data_get_overwrite (GrlData *data);
 
 G_END_DECLS
 
