@@ -71,6 +71,8 @@ GRL_LOG_DOMAIN_STATIC(test_ui_log_domain);
 
 #define WINDOW_TITLE "Grilo Test UI"
 
+#define NOTIFICATION_TIMEOUT 5
+
 #define BROWSER_MIN_WIDTH   320
 #define BROWSER_MIN_HEIGHT  400
 
@@ -129,9 +131,13 @@ typedef struct {
   GtkTreeModel *browser_model;
   GtkWidget *metadata;
   GtkTreeModel *metadata_model;
+  GtkWidget *statusbar;
+  guint statusbar_context_id;
 } UiView;
 
 typedef struct {
+  gboolean changes_notification;
+
   /* Keeps track of our browsing position and history  */
   GList *source_stack;
   GList *container_stack;
@@ -180,6 +186,7 @@ static const gchar *ui_definition =
 "  <menu name='FileMenu' action='FileMenuAction' >"
 "   <menuitem name='Authorize Flickr' action='AuthorizeFlickrAction' />"
 "   <menuitem name='Shutdown plugins' action='ShutdownPluginsAction' />"
+"   <menuitem name='Changes notification' action='ChangesNotificationAction' />"
 "   <menuitem name='Quit' action='QuitAction' />"
 "  </menu>"
 " </menubar>"
@@ -194,6 +201,13 @@ static void authorize_flickr_cb (GtkAction *action);
 static void shutdown_plugins_cb (GtkAction *action);
 static void shutdown_plugins (void);
 
+static void changes_notification_cb (GtkToggleAction *action);
+static void content_changed_cb (GrlMediaSource *source,
+                                GrlMedia *media,
+                                GrlMediaSourceChangeType change_type,
+                                gboolean location_unknown,
+                                gpointer data);
+
 static GtkActionEntry entries[] = {
   { "FileMenuAction", NULL, "_File" },
   { "AuthorizeFlickrAction", GTK_STOCK_CONNECT, "_Authorize Flickr", NULL,
@@ -204,6 +218,11 @@ static GtkActionEntry entries[] = {
     "Quit", G_CALLBACK (quit_cb) }
 };
 
+static GtkToggleActionEntry toggle_entries[] = {
+  { "ChangesNotificationAction", GTK_STOCK_FIND, "_Changes notification", NULL,
+    "ChangesNotification", G_CALLBACK (changes_notification_cb), FALSE }
+};
+
 static void
 quit_cb (GtkAction *action)
 {
@@ -222,6 +241,38 @@ shutdown_plugins_cb (GtkAction *action)
   shutdown_plugins ();
 }
 
+static void
+changes_notification_cb (GtkToggleAction *action)
+{
+  GList *sources, *source;
+  GrlPluginRegistry *registry;
+
+  ui_state->changes_notification = gtk_toggle_action_get_active (action);
+
+  registry = grl_plugin_registry_get_default ();
+  sources = grl_plugin_registry_get_sources (registry, FALSE);
+  for (source = sources; source; source = g_list_next (source)) {
+    if (grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source->data)) &
+        GRL_OP_NOTIFY_CHANGE) {
+      if (ui_state->changes_notification) {
+        grl_media_source_notify_change_start (GRL_MEDIA_SOURCE (source->data),
+                                              NULL);
+        g_signal_connect (GRL_MEDIA_SOURCE (source->data),
+                          "content-changed",
+                          G_CALLBACK (content_changed_cb),
+                          NULL);
+      } else {
+        grl_media_source_notify_change_stop (GRL_MEDIA_SOURCE (source->data),
+                                             NULL);
+        g_signal_handlers_disconnect_by_func (source->data,
+                                              content_changed_cb,
+                                              NULL);
+      }
+    }
+  }
+  g_list_free (sources);
+}
+
 static GtkTreeModel *
 create_browser_model (void)
 {
@@ -490,15 +541,18 @@ metadata_cb (GrlMediaSource *source,
     keys = grl_data_get_keys (GRL_DATA (media));
     i = keys;
     while (i) {
-      const GValue *g_value = grl_data_get (GRL_DATA (media), i->data);
-      gchar *value = g_value ? g_strdup_value_contents (g_value) : "";
-      gtk_list_store_append (GTK_LIST_STORE (view->metadata_model), &iter);
-      gtk_list_store_set (GTK_LIST_STORE (view->metadata_model),
-			  &iter,
-			  METADATA_MODEL_NAME, GRL_METADATA_KEY_GET_NAME (i->data),
-			  METADATA_MODEL_VALUE, value,
-			  -1);
-      GRL_DEBUG ("  %s: %s", GRL_METADATA_KEY_GET_NAME (i->data), value);
+      if (grl_data_has_key (GRL_DATA (media), i->data)) {
+        const GValue *g_value = grl_data_get (GRL_DATA (media), i->data);
+        gchar *value = g_value ? g_strdup_value_contents (g_value) : "";
+        gtk_list_store_append (GTK_LIST_STORE (view->metadata_model), &iter);
+        gtk_list_store_set (GTK_LIST_STORE (view->metadata_model),
+                            &iter,
+                            METADATA_MODEL_NAME,
+                            GRL_METADATA_KEY_GET_NAME (i->data),
+                            METADATA_MODEL_VALUE, value,
+                            -1);
+        GRL_DEBUG ("  %s: %s", GRL_METADATA_KEY_GET_NAME (i->data), value);
+      }
       i = g_list_next (i);
     }
 
@@ -1390,6 +1444,19 @@ set_vimeo_config (void)
   grl_plugin_registry_add_config (registry, config, NULL);
 }
 
+static void
+set_local_config (void)
+{
+  GrlConfig *config;
+  GrlPluginRegistry *registry;
+
+  config = grl_config_new ("grl-local-metadata", NULL);
+  grl_config_set_boolean (config, "guess-video", TRUE);
+
+  registry = grl_plugin_registry_get_default ();
+  grl_plugin_registry_add_config (registry, config, NULL);
+}
+
 static void
 launchers_setup (void)
 {
@@ -1423,6 +1490,7 @@ ui_setup (void)
 
   GtkActionGroup *actions = gtk_action_group_new ("actions");
   gtk_action_group_add_actions (actions, entries, G_N_ELEMENTS (entries), NULL);
+  gtk_action_group_add_toggle_actions (actions, toggle_entries, G_N_ELEMENTS (toggle_entries), NULL);
 
   GtkUIManager *uiman = gtk_ui_manager_new ();
   gtk_ui_manager_insert_action_group (uiman, actions, 0);
@@ -1604,6 +1672,17 @@ ui_setup (void)
 
   gtk_container_add (GTK_CONTAINER (scroll_md), view->metadata);
   gtk_container_add (GTK_CONTAINER (view->rpane), scroll_md);
+
+  /* Status bar */
+  view->statusbar = gtk_statusbar_new ();
+  gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (view->statusbar), FALSE);
+  view->statusbar_context_id =
+    gtk_statusbar_get_context_id (GTK_STATUSBAR (view->statusbar),
+                                  "changes notification");
+  gtk_container_add_with_properties (GTK_CONTAINER (view->rpane),
+                                     view->statusbar,
+                                     "expand", FALSE, NULL);
+
   gtk_widget_set_size_request (view->metadata,
 			       METADATA_MIN_WIDTH,
 			       METADATA_MIN_HEIGHT);
@@ -1685,6 +1764,16 @@ reset_ui (void)
   show_plugins ();
 }
 
+static gboolean
+remove_notification (gpointer data)
+{
+  gtk_statusbar_remove (GTK_STATUSBAR (view->statusbar),
+                        view->statusbar_context_id,
+                        GPOINTER_TO_UINT (data));
+
+  return FALSE;
+}
+
 static void
 content_changed_cb (GrlMediaSource *source,
                     GrlMedia *media,
@@ -1695,6 +1784,8 @@ content_changed_cb (GrlMediaSource *source,
   const gchar *media_id = grl_media_get_id (media);
   const gchar *change_type_string = "";
   const gchar *location_string = "";
+  gchar *message;
+  guint id;
 
   switch (change_type) {
   case GRL_CONTENT_CHANGED:
@@ -1709,21 +1800,32 @@ content_changed_cb (GrlMediaSource *source,
   }
 
   if (location_unknown) {
-    location_string = ", can identify exactly where";
+    location_string = "(unknown place)";
   }
 
   if (GRL_IS_MEDIA_BOX (media)) {
-    GRL_DEBUG ("Content changed in %s: something has %s in container '%s'%s",
-               grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
-               change_type_string,
-               media_id? media_id: "root",
-               location_string);
+    message =
+      g_strdup_printf ("%s: container '%s' has %s%s",
+                       grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
+                       media_id? media_id: "root",
+                       change_type_string,
+                       location_string);
   } else {
-    GRL_DEBUG ("Content changed in %s: element '%s' has %s",
-               grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
-               media_id,
-               change_type_string);
+    message =
+      g_strdup_printf ("%s: element '%s' has %s",
+                       grl_metadata_source_get_name (GRL_METADATA_SOURCE (source)),
+                       media_id,
+                       change_type_string);
   }
+
+  id = gtk_statusbar_push (GTK_STATUSBAR (view->statusbar),
+                           view->statusbar_context_id,
+                           message);
+
+  g_timeout_add_seconds (NOTIFICATION_TIMEOUT,
+                         remove_notification,
+                         GUINT_TO_POINTER (id));
+  g_free (message);
 }
 
 static void
@@ -1751,7 +1853,8 @@ source_added_cb (GrlPluginRegistry *registry,
   query_combo_setup ();
 
   /* Check for changes in source (if supported) */
-  if ((grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source)) &
+  if (ui_state->changes_notification &&
+      (grl_metadata_source_supported_operations (GRL_METADATA_SOURCE (source)) &
        GRL_OP_NOTIFY_CHANGE)) {
     if (grl_media_source_notify_change_start (GRL_MEDIA_SOURCE (source), NULL)) {
       g_signal_connect (GRL_MEDIA_SOURCE (source), "content-changed",
@@ -1847,6 +1950,7 @@ configure_plugins ()
   set_flickr_config ();
   set_youtube_config ();
   set_vimeo_config ();
+  set_local_config ();
 }
 
 int
